@@ -1,6 +1,6 @@
 /* filehash:  Print various hash digests and filesizes for specified files.
 
-   Copyright (C) 2004-2020 by Brian Lindholm.
+   Copyright (C) 2004-2021 by Brian Lindholm.
    This file is part of the littleutils utility set.
 
    The filehash utility is free software; you can redistribute it and/or modify
@@ -19,33 +19,34 @@
 
 #include <config.h>
 
+#if HAVE_INTTYPES_H
+# include <inttypes.h>
+#endif
 #include <limits.h>
-#include <stdio.h>
+#ifdef HAVE_STDIO_H
+# include <stdio.h>
+#endif
 #ifdef HAVE_STDLIB_H
 # include <stdlib.h>
 #endif
 #ifdef HAVE_STRING_H
 # include <string.h>
 #endif
-#ifdef HAVE_SYS_TYPES_H
-# include <sys/types.h>
-#endif
 #ifdef HAVE_SYS_STAT_H
 # include <sys/stat.h>
 #endif
-
-#ifdef HAVE_GETOPT_H
-# include <getopt.h>
+#ifdef HAVE_SYS_TYPES_H
+# include <sys/types.h>
 #endif
+
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 # define OPTEND -1
 #else
 # define OPTEND EOF
 #endif
-
-#if HAVE_INTTYPES_H
-# include <inttypes.h>
+#ifdef HAVE_GETOPT_H
+# include <getopt.h>
 #endif
 
 #ifdef MSDOS
@@ -86,22 +87,61 @@ help (FILE *where)
     "filehash " PACKAGE_VERSION "\n"
     "usage: filehash [-1(MD5)] [-2(SHA1)] [-3(SHA224) ][-4(SHA256)]\n"
     "         [-5(SHA384)] [-6(SHA512)] [-7(BLAKE2B_256)] [-8(BLAKE2B_256)]\n"
-    "         [-c(lassic)] [-f file_list] [-h(elp)] [-n byte_count]\n"
-    "         [-o offset] [-p(ipe) ] [-q(uiet)] [-s(ize)] file...\n");
+    "         [-b(ase64url)] [-c(lassic)] [-f file_list] [-h(elp)]\n"
+    "         [-n byte_count] [-o offset] [-p(ipe) ] [-q(uiet)] [-s(ize)]\n"
+    "         [-v(erbose)] file...\n");
+}
+
+
+static void
+encode_hash (unsigned char *string, unsigned char *hash, int bytes, int use_base64)
+{
+  int i, j;
+  static const unsigned char base64_table[65] =
+    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_";
+  static const unsigned char hex_table[65] = "0123456789abcdef";
+
+  j = 0;
+  if (use_base64) {
+    for (i = 0; i < (bytes / 3); ++i) {
+      string[j++] = base64_table[hash[i*3] >> 2];
+      string[j++] = base64_table[((hash[i*3] & 0x03) << 4) | (hash[i*3+1] >> 4)];
+      string[j++] = base64_table[((hash[i*3+1] & 0x0F) << 2) | (hash[i*3+2] >> 6)];
+      string[j++] = base64_table[hash[i*3+2] & 0x3F];
+    }
+    if ((bytes % 3) == 2) {
+      string[j++] = base64_table[hash[bytes-2] >> 2];
+      string[j++] = base64_table[((hash[bytes-2] & 0x03) << 4) | (hash[bytes-1] >> 4)];
+      string[j++] = base64_table[(hash[bytes-1] & 0x0F) << 2];
+    }
+    else if ((bytes % 3) == 1) {
+      string[j++] = base64_table[hash[bytes-1] >> 2];
+      string[j++] = base64_table[(hash[bytes-1] & 0x03) << 4];
+    }
+    string[j++] = '\000';
+  }
+  else {
+    for (i = 0; i < bytes; ++i) {
+      string[j++] = hex_table[hash[i] >> 4];
+      string[j++] = hex_table[hash[i] & 0x0F];
+    }
+    string[j++] = '\000';
+  }
 }
 
 
 static void
 print_filehash (char *filename, off_t offset, off_t read_bytes, int print_size,
   int run_md5, int run_sha1, int run_sha224, int run_sha256, int run_sha384,
-  int run_sha512, int run_blake2b_256, int run_blake2b_512, int verbose, int classic)
+  int run_sha512, int run_blake2b_256, int run_blake2b_512, int classic,
+  int use_base64, int verbose)
 {
   FILE *file;
   int i, not_first, rc;
   struct stat file_stats;
   unsigned char md5result[16], sha1result[20], sha224result[28],
                 sha256result[32], sha384result[48], sha512result[64],
-                blake2b_256result[32], blake2b_512result[64];
+                blake2b_256result[32], blake2b_512result[64], string[129];
 
   if (stat (filename, &file_stats))
     fprintf (stderr, "filehash error: can't stat %s\n", filename);
@@ -118,8 +158,7 @@ print_filehash (char *filename, off_t offset, off_t read_bytes, int print_size,
               (void) fseek (file, (off_t) offset, 0);
             rc = md5_stream (file, md5result, read_bytes);
             if (rc)
-              fprintf (stderr,
-                "filehash error: md5_stream failed on %s\n", filename);
+              fprintf (stderr, "filehash error: md5_stream failed on %s\n", filename);
             not_first = 1;
           }
           if (run_sha1) {
@@ -127,8 +166,7 @@ print_filehash (char *filename, off_t offset, off_t read_bytes, int print_size,
               (void) fseek (file, (off_t) offset, 0);
             rc = sha1_stream (file, sha1result, read_bytes);
             if (rc)
-              fprintf (stderr,
-                "filehash error: sha1_stream failed on %s\n", filename);
+              fprintf (stderr, "filehash error: sha1_stream failed on %s\n", filename);
             not_first = 1;
           }
           if (run_sha224) {
@@ -136,8 +174,7 @@ print_filehash (char *filename, off_t offset, off_t read_bytes, int print_size,
               (void) fseek (file, (off_t) offset, 0);
             rc = sha224_stream (file, sha224result, read_bytes);
             if (rc)
-              fprintf (stderr,
-                "filehash error: sha224_stream failed on %s\n", filename);
+              fprintf (stderr, "filehash error: sha224_stream failed on %s\n", filename);
             not_first = 1;
           }
           if (run_sha256) {
@@ -145,8 +182,7 @@ print_filehash (char *filename, off_t offset, off_t read_bytes, int print_size,
               (void) fseek (file, (off_t) offset, 0);
             rc = sha256_stream (file, sha256result, read_bytes);
             if (rc)
-              fprintf (stderr,
-                "filehash error: sha256_stream failed on %s\n", filename);
+              fprintf (stderr, "filehash error: sha256_stream failed on %s\n", filename);
             not_first = 1;
           }
           if (run_sha384) {
@@ -154,8 +190,7 @@ print_filehash (char *filename, off_t offset, off_t read_bytes, int print_size,
               (void) fseek (file, (off_t) offset, 0);
             rc = sha384_stream (file, sha384result, read_bytes);
             if (rc)
-              fprintf (stderr,
-                "filehash error: sha384_stream failed on %s\n", filename);
+              fprintf (stderr, "filehash error: sha384_stream failed on %s\n", filename);
             not_first = 1;
           }
           if (run_sha512) {
@@ -163,8 +198,7 @@ print_filehash (char *filename, off_t offset, off_t read_bytes, int print_size,
               (void) fseek (file, (off_t) offset, 0);
             rc = sha512_stream (file, sha512result, read_bytes);
             if (rc)
-              fprintf (stderr,
-                "filehash error: sha512_stream failed on %s\n", filename);
+              fprintf (stderr, "filehash error: sha512_stream failed on %s\n", filename);
             not_first = 1;
           }
           if (run_blake2b_256) {
@@ -172,8 +206,7 @@ print_filehash (char *filename, off_t offset, off_t read_bytes, int print_size,
               (void) fseek (file, (off_t) offset, 0);
             rc = blake2b_stream (file, blake2b_256result, 32, read_bytes);
             if (rc)
-              fprintf (stderr,
-                "filehash error: sha384_stream failed on %s\n", filename);
+              fprintf (stderr, "filehash error: sha384_stream failed on %s\n", filename);
             not_first = 1;
           }
           if (run_blake2b_512) {
@@ -181,8 +214,7 @@ print_filehash (char *filename, off_t offset, off_t read_bytes, int print_size,
               (void) fseek (file, (off_t) offset, 0);
             rc = blake2b_stream (file, blake2b_512result, 64, read_bytes);
             if (rc)
-              fprintf (stderr,
-                "filehash error: sha512_stream failed on %s\n", filename);
+              fprintf (stderr, "filehash error: sha512_stream failed on %s\n", filename);
             not_first = 1;
           }
           (void) fclose (file);
@@ -199,117 +231,96 @@ print_filehash (char *filename, off_t offset, off_t read_bytes, int print_size,
             else {
               if (not_first)
                 fprintf (stdout, "\t");
-              fprintf (stdout, sizefmt, file_stats.st_size);
               not_first = 1;
+              fprintf (stdout, sizefmt, file_stats.st_size);
             }
           }
           if (run_md5) {
-            if (classic) {
-              for (i = 0; i < 16; ++i)
-                fprintf (stdout, "%02x", (unsigned int) md5result[i]);
-              fprintf (stdout, "  %s\n", filename);
-            }
+            encode_hash(string, md5result, 16, use_base64);
+            if (classic)
+              fprintf (stdout, "%s  %s\n", string, filename);
             else {
               if (not_first)
                 fprintf (stdout, "\t");
-              for (i = 0; i < 16; ++i)
-                fprintf (stdout, "%02x", (unsigned int) md5result[i]);
               not_first = 1;
+              fprintf (stdout, "%s", string);
             }
           }
           if (run_sha1) {
-            if (classic) {
-              for (i = 0; i < 20; ++i)
-                fprintf (stdout, "%02x", (unsigned int) sha1result[i]);
-              fprintf (stdout, "  %s\n", filename);
-            }
+            encode_hash(string, sha1result, 20, use_base64);
+            if (classic)
+              fprintf (stdout, "%s  %s\n", string, filename);
             else {
               if (not_first)
                 fprintf (stdout, "\t");
-              for (i = 0; i < 20; ++i)
-                fprintf (stdout, "%02x", (unsigned int) sha1result[i]);
               not_first = 1;
+              fprintf (stdout, "%s", string);
             }
           }
           if (run_sha224) {
-            if (classic) {
-              for (i = 0; i < 28; ++i)
-                fprintf (stdout, "%02x", (unsigned int) sha224result[i]);
-              fprintf (stdout, "  %s\n", filename);
-            }
+            encode_hash(string, sha224result, 28, use_base64);
+            if (classic)
+              fprintf (stdout, "%s  %s\n", string, filename);
             else {
               if (not_first)
                 fprintf (stdout, "\t");
-              for (i = 0; i < 28; ++i)
-                fprintf (stdout, "%02x", (unsigned int) sha224result[i]);
               not_first = 1;
+              fprintf (stdout, "%s", string);
             }
           }
           if (run_sha256) {
-            if (classic) {
-              for (i = 0; i < 32; ++i)
-                fprintf (stdout, "%02x", (unsigned int) sha256result[i]);
-              fprintf (stdout, "  %s\n", filename);
-            }
+            encode_hash(string, sha256result, 32, use_base64);
+            if (classic)
+              fprintf (stdout, "%s  %s\n", string, filename);
             else {
               if (not_first)
                 fprintf (stdout, "\t");
-              for (i = 0; i < 32; ++i)
-                fprintf (stdout, "%02x", (unsigned int) sha256result[i]);
               not_first = 1;
+              fprintf (stdout, "%s", string);
             }
           }
           if (run_sha384) {
-            if (classic) {
-              for (i = 0; i < 48; ++i)
-                fprintf (stdout, "%02x", (unsigned int) sha384result[i]);
-              fprintf (stdout, "  %s\n", filename);
-            }
+            encode_hash(string, sha384result, 48, use_base64);
+            if (classic)
+              fprintf (stdout, "%s  %s\n", string, filename);
             else {
               if (not_first)
                 fprintf (stdout, "\t");
-              for (i = 0; i < 48; ++i)
-                fprintf (stdout, "%02x", (unsigned int) sha384result[i]);
               not_first = 1;
+              fprintf (stdout, "%s", string);
             }
           }
           if (run_sha512) {
-            if (classic) {
-              for (i = 0; i < 64; ++i)
-                fprintf (stdout, "%02x", (unsigned int) sha512result[i]);
-              fprintf (stdout, "  %s\n", filename);
-            }
+            encode_hash(string, sha512result, 64, use_base64);
+            if (classic)
+              fprintf (stdout, "%s  %s\n", string, filename);
             else {
               if (not_first)
                 fprintf (stdout, "\t");
-              for (i = 0; i < 64; ++i)
-                fprintf (stdout, "%02x", (unsigned int) sha512result[i]);
+              not_first = 1;
+              fprintf (stdout, "%s", string);
             }
           }
           if (run_blake2b_256) {
-            if (classic) {
-              for (i = 0; i < 32; ++i)
-                fprintf (stdout, "%02x", (unsigned int) blake2b_256result[i]);
-              fprintf (stdout, "  %s\n", filename);
-            }
+            encode_hash(string, blake2b_256result, 32, use_base64);
+            if (classic)
+              fprintf (stdout, "%s  %s\n", string, filename);
             else {
               if (not_first)
                 fprintf (stdout, "\t");
-              for (i = 0; i < 32; ++i)
-                fprintf (stdout, "%02x", (unsigned int) blake2b_256result[i]);
+              not_first = 1;
+              fprintf (stdout, "%s", string);
             }
           }
           if (run_blake2b_512) {
-            if (classic) {
-              for (i = 0; i < 64; ++i)
-                fprintf (stdout, "%02x", (unsigned int) blake2b_512result[i]);
-              fprintf (stdout, "  %s\n", filename);
-            }
+            encode_hash(string, blake2b_512result, 64, use_base64);
+            if (classic)
+              fprintf (stdout, "%s  %s\n", string, filename);
             else {
               if (not_first)
                 fprintf (stdout, "\t");
-              for (i = 0; i < 64; ++i)
-                fprintf (stdout, "%02x", (unsigned int) blake2b_512result[i]);
+              not_first = 1;
+              fprintf (stdout, "%s", string);
             }
           }
           if (classic == 0)
@@ -326,7 +337,7 @@ main (int argc, char **argv)
   char filename[PATH_MAX], *listname, *newline, *rc;
   int argn, classic, offset, opt, print_size, run_md5, run_sha1, run_sha224,
       run_sha256, run_sha384, run_sha512, run_blake2b_256, run_blake2b_512,
-      use_file, use_pipe, verbose;
+      use_base64, use_file, use_pipe, verbose;
   off_t read_bytes, tmp_bytes;
 
   /* parse options */
@@ -344,10 +355,11 @@ main (int argc, char **argv)
   run_sha512 = 0;
   run_blake2b_256 = 0;
   run_blake2b_512 = 0;
+  use_base64 = 0;
   use_file = 0;
   use_pipe = 0;
   verbose = 0;
-  while ((opt = getopt (argc, argv, "12345678cf:hpn:o:qsv")) != OPTEND)
+  while ((opt = getopt (argc, argv, "12345678bcf:hpn:o:qsv")) != OPTEND)
     switch (opt)
       {
       case '1':
@@ -374,6 +386,9 @@ main (int argc, char **argv)
       case '8':
         run_blake2b_512 = 1;
         break;
+      case 'b':
+        use_base64 = 1;
+        break;
       case 'c':
         classic = 1;
         break;
@@ -453,8 +468,8 @@ main (int argc, char **argv)
                   if (strlen (filename) != 0)
                     print_filehash (filename, offset, read_bytes, print_size,
                       run_md5, run_sha1, run_sha224, run_sha256, run_sha384,
-                      run_sha512, run_blake2b_256, run_blake2b_512, verbose,
-                      classic);
+                      run_sha512, run_blake2b_256, run_blake2b_512, classic,
+                      use_base64, verbose);
                 }
             }
           (void) fclose (infile);
@@ -473,19 +488,18 @@ main (int argc, char **argv)
             if (newline != NULL)
               *newline = '\0';
             if (strlen (filename) != 0)
-              print_filehash (filename, offset, read_bytes, print_size,
-                run_md5, run_sha1, run_sha224, run_sha256, run_sha384,
-                run_sha512, run_blake2b_256, run_blake2b_512, verbose,
-                classic);
+              print_filehash (filename, offset, read_bytes, print_size, run_md5,
+                run_sha1, run_sha224, run_sha256, run_sha384, run_sha512,
+                run_blake2b_256, run_blake2b_512, classic, use_base64, verbose);
           }
       }
 
   /* process files given in the argument list */
 
   for (argn = optind; argn < argc; argn++)
-    print_filehash (argv[argn], offset, read_bytes, print_size,
-      run_md5, run_sha1, run_sha224, run_sha256, run_sha384,
-      run_sha512, run_blake2b_256, run_blake2b_512, verbose, classic);
+    print_filehash (argv[argn], offset, read_bytes, print_size, run_md5,
+      run_sha1, run_sha224, run_sha256, run_sha384, run_sha512,
+      run_blake2b_256, run_blake2b_512, classic, use_base64, verbose);
 
   /* indicate successful finish */
 
