@@ -1,6 +1,6 @@
 /* filesize:  Prints out the file sizes of the specified files.
 
-   Copyright (C) 2004-2020 by Brian Lindholm.
+   Copyright (C) 2004-2021 by Brian Lindholm.
    This file is part of the littleutils utility set.
 
    The filesize utility is free software; you can redistribute it and/or modify
@@ -19,8 +19,13 @@
 
 #include <config.h>
 
+#if HAVE_INTTYPES_H
+# include <inttypes.h>
+#endif
 #include <limits.h>
-#include <stdio.h>
+#ifdef HAVE_STDIO_H
+# include <stdio.h>
+#endif
 #ifdef HAVE_STDLIB_H
 # include <stdlib.h>
 #endif
@@ -31,18 +36,14 @@
 # include <sys/stat.h>
 #endif
 
-#ifdef HAVE_GETOPT_H
-# include <getopt.h>
-#endif
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 # define OPTEND -1
 #else
 # define OPTEND EOF
 #endif
-
-#if HAVE_INTTYPES_H
-# include <inttypes.h>
+#ifdef HAVE_GETOPT_H
+# include <getopt.h>
 #endif
 
 #ifdef __MINGW32__
@@ -61,6 +62,7 @@ unsigned short _djstat_flags = 63;  /* speed up stat command for DJGPP */
 
 char *sizefmt_q = (sizeof (off_t) <= sizeof (long) ? "%lu\n" : "%llu\n");
 char *sizefmt_v = (sizeof (off_t) <= sizeof (long) ? "%s\t%lu\n" : "%s\t%llu\n");
+off_t total_size = 0;
 
 
 /* help function */
@@ -71,14 +73,14 @@ help (FILE *where)
   fprintf (where,
     "filesize " PACKAGE_VERSION "\n"
     "usage: filesize [-f file_list] [-h(elp)] [-p(ipe)] [-q(uiet)]\n"
-    "         [-v(erbose)] file...\n");
+    "         [-t(otalize)] [-v(erbose)] file...\n");
 }
 
 
 /* print filesize function */
 
 static void
-printsize (char *filename, int verbose)
+printsize (char *filename, int totalize, int verbose)
 {
   struct stat file_stats;
 
@@ -88,7 +90,9 @@ printsize (char *filename, int verbose)
     if (((file_stats.st_mode & S_IFDIR) != S_IFDIR) &&
         ((file_stats.st_mode & S_IFREG) == S_IFREG))
       {
-        if (verbose == 1)
+        if (totalize == 1)
+          total_size += file_stats.st_size;
+        else if (verbose == 1)
           fprintf (stdout, sizefmt_v, filename, file_stats.st_size);
         else
           fprintf (stdout, sizefmt_q, file_stats.st_size);
@@ -103,15 +107,16 @@ main (int argc, char *argv[])
 {
   FILE *infile;
   char filename[PATH_MAX], *listname, *newline, *rc;
-  int argn, opt, use_file, use_pipe, verbose;
+  int argn, opt, totalize, use_file, use_pipe, verbose;
 
   /* parse options */
 
   listname = "";
+  totalize = 0;
   verbose = 0;
   use_file = 0;
   use_pipe = 0;
-  while ((opt = getopt (argc, argv, "f:hpqv")) != OPTEND)
+  while ((opt = getopt (argc, argv, "f:hpqtv")) != OPTEND)
     switch (opt)
       {
       case 'f':
@@ -127,6 +132,9 @@ main (int argc, char *argv[])
       case 'q':
         verbose = -1;
         break;
+      case 't':
+        totalize = 1;
+        break;
       case 'v':
         verbose = 1;
         break;
@@ -168,7 +176,7 @@ main (int argc, char *argv[])
                   if (newline != NULL)
                     *newline = '\0';
                   if (strlen (filename) != 0)
-                    printsize (filename, verbose);
+                    printsize (filename, totalize, verbose);
                 }
             }
           (void) fclose (infile);
@@ -187,14 +195,19 @@ main (int argc, char *argv[])
             if (newline != NULL)
               *newline = '\0';
             if (strlen (filename) != 0)
-              printsize (filename, verbose);
+              printsize (filename, totalize, verbose);
           }
       }
 
   /* process files given in the argument list */
 
   for (argn = optind; argn < argc; argn++)
-    printsize (argv[argn], verbose);
+    printsize (argv[argn], totalize, verbose);
+
+  /* print total size if requested */
+
+  if (totalize == 1)
+    fprintf (stdout, sizefmt_q, total_size);
 
   /* indicate successful finish */
 
