@@ -20,32 +20,35 @@
 
 #include <config.h>
 
-#include <stdio.h>
+#include <errno.h>
+#include <fcntl.h>
+#ifdef HAVE_STDIO_H
+# include <stdio.h>
+#endif
 #ifdef HAVE_STDLIB_H
 # include <stdlib.h>
 #endif
 #ifdef HAVE_STRING_H
 # include <string.h>
 #endif
-#include <time.h>
-#ifdef HAVE_SYS_TYPES_H
-# include <sys/types.h>
-#endif
 #ifdef HAVE_SYS_STAT_H
 # include <sys/stat.h>
 #endif
-#include <fcntl.h>
-#include <errno.h>
-
-#ifdef HAVE_GETOPT_H
-#include <getopt.h>
+#ifdef HAVE_SYS_TYPES_H
+# include <sys/types.h>
 #endif
+
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 # define OPTEND -1
 #else
 # define OPTEND EOF
 #endif
+#ifdef HAVE_GETOPT_H
+# include <getopt.h>
+#endif
+
+#include "rand_funcs.h"
 
 #ifdef __MINGW32__
 extern int getopt (int argc, char * const *argv, const char *optstring);
@@ -142,49 +145,6 @@ good_dir (const char *path, int verbose)
 }
 
 
-/* random seed function */
-
-static void
-rand_seed ()
-{
-  int URANDOM, urandom_worked = 1;
-  ssize_t bytes;
-  time_t junk;
-  unsigned short seed[3];
-
-  /* pre-populate array for seed using /dev/urandom */
-  URANDOM = open("/dev/urandom", O_RDONLY);
-  if (URANDOM < 0) {
-    urandom_worked = 0;
-  }
-  else {
-    /* use read instead of buffered fread to avoid grabbing extra bytes */
-    bytes = read(URANDOM, seed, sizeof seed);
-    if (bytes < 6) {
-      urandom_worked = 0;
-    }
-  }
-  close(URANDOM);
-
-  /* fall back to time and PID combo if /dev/urandom failed */
-  if (urandom_worked == 0) {
-    (void) time (&junk);
-    seed[0] = (unsigned short) (junk & 0xFFFF);
-    seed[1] = (unsigned short) (getpid () & 0xFFFF);
-    seed[2] = (unsigned short) ((junk >> 16) & 0xFFFF);
-  }
-
-  /* initialize random number generator seed */
-#ifdef HAVE_LRAND48
-  (void) seed48 (seed);
-#elif defined(HAVE_RANDOM)
-  srandom (((unsigned int) seed[0] << 16) + (unsigned int) seed[1]);
-#else
-  srand (((unsigned int) seed[0] << 16) + (unsigned int) seed[1]);
-#endif
-}
-
-
 /* special variant of mkstemp, allows X's to occur in non-end locations */
 
 static int
@@ -429,7 +389,7 @@ main (int argc, char *argv[])
   if (add_wild)
     strcat (newname, "_*");
   else if (add_random)
-    strcat (newname, "_||||||");  /* use | instead of X in case path contains X */
+    strcat (newname, "_||||||||");  /* use | instead of X in case path contains X */
   if (add_suffix)
     strcat (newname, suffix);
 
